@@ -129,7 +129,7 @@ stsm_detect_breaks = function(model, y, components = c("trend", "cycle", "season
   
   #Filter and smooth the data
   ssm = stsm_ssm(yt = y, model = model)
-  B_tt = kalman_filter(ssm, matrix(y, nrow = 1), X, smooth)$B_tt
+  msg = utils::capture.output(B_tt <- kalman_filter(ssm, matrix(y, nrow = 1), X, smooth)$B_tt, type = "message")
   rownames(B_tt) = rownames(ssm$Fm)
   
   #Get the unobserved series
